# Fit the univariate latent regression model with a skew-normal error:
#  y = beta0 + beta1*x + epsilon,
# where x has a beta distribution and epsilon is skew-normal.
#  You must load the skew-normal sn library for this program.
#  The "soln" function from R is used to find the mle's.
# 
# This program reads in the week 2 improvement in depression based on the
#  HAM-D differences for depressed subjects treated with Prozac.

source("emnorm.r")  # emnorm is a routine that fits a finite mixture model.
#library(mvtnorm)
library(sn)

# Read in the data and put the response into the vector y:
y <- read.table("HamDdifferencesWeek2.dat")
y <- as.matrix(y[,2])

n=dim(y)[1]
ab=matrix(0,2,1)
bhat=matrix(0,2,1)

# Provide initial values for the parameters:
bhat[1,1]= -2.2256725
bhat[2,1]=  9.3590143
ab[1,1]=  0.2060662
ab[2,1]=  0.2845610
s=  7.4952008
alpha=  3.2483954

########################################

# Join all the parameters together in a single vector theta
theta=matrix(c(bhat[1,1],bhat[2,1],ab[1,1],ab[2,1],s,alpha),6,1)

# Define the loglikelihood function:
loglike <- function(theta)
{
theta <- as.matrix(theta)
y <- as.matrix(y)
kb <- gamma(theta[3,1]+theta[4,1])/(gamma(theta[3,1])*gamma(theta[4,1])) # beta normalization constants
fi=NULL
for(i in 1:n)
{
yi = y[i,1]

fy<- function(x){
     2*dnorm(yi, mean=(theta[1,1] +theta[2,1]*x), sd=theta[5,1], log = FALSE)*
       pnorm((yi-theta[1,1]-theta[2,1]*x)*theta[6,1]/theta[5,1])*kb*
             x^(theta[3,1]-1)*(1-x)^(theta[4,1]-1)}

fii=integrate(fy,0,1)
fi=rbind(fi,fii$value)
} 
return( sum(log(fi)))
}


# Use the built-in R function "optim" to find the mle's:
soln <- optim(theta, loglike, gr=NULL, method="L-BFGS-B", 
              control=list(fnscale=-1),#  , maxit=50),
              lower=c(-Inf,0,0.1,0.1,0,0), upper=c(Inf,Inf,Inf,Inf,Inf,Inf),
              hessian=T)
theta <- as.matrix(soln$par)
llike <- soln$value
bhat=as.matrix(theta[1:2,])
ab=as.matrix(theta[3:4,])
s=as.numeric(theta[5,1])
alpha=as.numeric(theta[6,1])


# Fit the finite mixture model
k  <-  2                      # k = number of mixture components
prior  <-  matrix(1,k)/k      # Define a vector of initial values for the prior probabilities
pool   <-  0                  #  indicator for equality of variances: pool=0: unequal variances; pool=1: equal variances
S <- cov(y)
p <- dim(y)[2]
# Run k-means to get initial values for mixture component means
kmean  <-  kmeans(y, k, iter.max = 100)
mu <- as.matrix(sort(kmean$center))      # Initial component means as a k x p matrix
var <-  array(S, c(p,p, k))   # Initial covariance matrix estimates
results <- emnorm(k,y, prior, mu, var, pool)
muvar<- t(matrix(cbind(results$prior,results$mu,results$var),6,1))

shapiro.test(y)

par(mfrow=c(2,1))
dy=density(y, bw=2.5)
yd=as.matrix(seq(-10,40, by=.1))
fy2=NULL
for (ij in 1:length(yd)){
int2 <- function(x){2*
dnorm(yd[ij,1], mean=(bhat[1,1]+bhat[2,1]*x),sd=s,log = FALSE)*
     pnorm((yd[ij,1]-bhat[1,1]-bhat[2,1]*x)*alpha/s)*
     gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1]))*
     x^(ab[1,1]-1)*(1-x)^(ab[2,1]-1)}
fy12 <- integrate(int2,0,1)$value
fy2 = rbind(fy2, fy12)}
hist(y,  25, freq=FALSE, ylim=c(0,0.1), main="Baseline-(Week 2) Ham-D",
xlab="Difference")
lines(dy, lwd=2)
lines(yd, fy2, lty=2,col=2, lwd=2)   # Latent Regression Fit
t <- seq(0,1, by = .005)
bt=gamma(ab[1,1]+ab[2,1])/(gamma(ab[1,1])*gamma(ab[2,1]))*
        t^(ab[1,1]-1)*(1-t)^(ab[2,1]-1)
plot (t,bt, type="l", main="Latent Beta Density",
           xlab="x", ylab="", lwd=3)


